
# q-expansion of newform 624.4.bv.h, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 624
weight = 4
poly_data = [27648, 0, 44100, 0, 14700, 0, 1645, 0, 70, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 4, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[96, 210, 0, 35, 0, 1, 0, 0, 0, 0], 192], [[0, 96, 210, 0, 35, 0, 1, 0, 0, 0], 48], [[28, 0, 2, 0, 0, 0, 0, 0, 0, 0], 1], [[1344, 2940, 96, 700, 0, 49, 0, 1, 0, 0], 96], [[24192, 15336, 13392, 9030, 1128, 1309, 24, 64, 0, 1], 1152], [[-24192, 15336, -13392, 9030, -1128, 1309, -24, 64, 0, 1], 1152], [[48384, 20304, 42840, 12654, 7008, 1603, 366, 70, 6, 1], 1152], [[0, -20304, 0, -12654, 0, -1603, 0, -70, 0, -1], 576]]

hecke_ring_character_values = [[79, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [469, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [209, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [145, [0, 0, 1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [3, 0, -3, 0, 0, 0, 0, 0, 0, 0], [1, 0, -2, 0, 0, 0, -1, -1, 0, 0], [-4, 0, 2, 0, 0, 0, 0, 0, -1, -1], [-4, -1, -4, 1, 1, 1, 1, 0, 1, 0], [11, 2, -17, -1, -1, 1, 2, 1, 1, 0], [0, -2, 21, 1, 0, 0, 0, 0, 1, -1], [-24, 0, 12, 3, -4, 2, 0, 3, 1, 1], [12, 0, -12, 0, 0, 0, -1, -2, -3, -6], [-99, 2, 99, 2, 3, -3, -3, -6, 0, 0], [22, 3, -44, 0, 3, -6, 3, 3, 0, -2], [-27, -6, -27, 6, 0, 0, 12, 0, -1, 0], [71, 10, 71, -10, 3, 3, 8, 0, -3, 0], [0, -4, 74, 2, 0, 3, 20, 10, 1, -1], [-4, 11, 8, 0, 2, -4, -3, -3, 0, -1], [33, 3, 0, -6, 3, 0, 9, -9, -8, -4], [-104, 0, 52, -5, 16, -8, 0, 6, -4, -4], [0, 6, -275, -3, 0, -2, 6, 3, -1, 1], [-106, -6, -106, 6, -7, -7, 12, 0, -3, 0], [-216, 0, 108, -24, 4, -2, 0, -1, 5, 5], [-57, 18, 114, 0, 5, -10, 0, 0, 0, 0], [-110, 19, 0, -38, 1, 0, 13, -13, -4, -2], [172, 18, -344, 0, 5, -10, 37, 37, 0, -19], [136, -31, 136, 31, -2, -2, -6, 0, 10, 0], [-500, 0, 250, 21, 32, -16, 0, 3, 22, 22], [-57, 31, 57, 31, -12, 12, -7, -14, -2, -4], [610, 16, 0, -32, 0, 0, 16, -16, 2, 1], [252, 30, -252, 30, -9, 9, -19, -38, 11, 22], [-154, -63, 308, 0, -5, 10, 27, 27, 0, -46], [0, 2, -507, -1, 0, 33, -20, -10, 9, -9], [-598, -13, 598, -13, -25, 25, -1, -2, 24, 48], [36, -44, 0, 88, 0, 0, -32, 32, -44, -22], [-1662, 0, 831, -47, 10, -5, 0, 14, -43, -43], [0, -18, 662, 9, 0, 33, -6, -3, 48, -48], [562, 0, -281, 50, 22, -11, 0, 3, 2, 2], [24, 63, -48, 0, 31, -62, 81, 81, 0, 3], [569, -27, 0, 54, -21, 0, -39, 39, 50, 25], [708, 0, -354, 3, -26, 13, 0, -51, -74, -74], [-680, -12, -680, 12, -28, -28, -34, 0, -10, 0], [0, -18, 726, 9, 0, 21, 92, 46, 34, -34], [1284, -78, -1284, -78, 63, -63, -1, -2, -5, -10], [839, 70, 0, -140, -3, 0, -80, 80, 42, 21], [0, 104, -60, -52, 0, -72, -156, -78, -20, 20], [193, -78, 193, 78, 50, 50, -24, 0, -66, 0], [164, 35, 164, -35, 50, 50, 96, 0, -124, 0], [0, -140, -190, 70, 0, -76, -80, -40, 27, -27], [-1522, -19, 1522, -19, 11, -11, -79, -158, -2, -4], [228, -18, 228, 18, 50, 50, 198, 0, 4, 0], [1200, 0, -600, -12, -122, 61, 0, 47, -49, -49], [148, -174, -296, 0, -5, 10, -102, -102, 0, -2], [906, -55, 0, 110, 102, 0, 56, -56, -120, -60], [-1356, 43, 2712, 0, -52, 104, -61, -61, 0, -85], [-2046, 0, 1023, 183, -60, 30, 0, -69, -70, -70], [0, -34, 1212, 17, 0, 48, 156, 78, -22, 22], [-369, 42, 369, 42, -186, 186, 10, 20, -59, -118], [-1656, -34, 1656, -34, -174, 174, -11, -22, 37, 74], [0, -36, -1002, 18, 0, 33, -304, -152, -50, 50], [-22, -105, -22, 105, 1, 1, -159, 0, -24, 0], [0, -12, 3979, 6, 0, -129, 12, 6, -49, 49], [-745, 198, 1490, 0, -53, 106, 206, 206, 0, 13], [-742, 94, 742, 94, 155, -155, -206, -412, 47, 94], [70, 0, -35, 61, -184, 92, 0, -85, 82, 82], [1854, -54, -3708, 0, -115, 230, 198, 198, 0, -45], [2544, -88, 0, 176, 198, 0, 17, -17, -90, -45], [2384, 53, 0, -106, 152, 0, 137, -137, -188, -94], [695, -343, -1390, 0, 108, -216, -167, -167, 0, -6], [-3420, 0, 1710, -153, 6, -3, 0, 81, -140, -140], [209, -4, 0, 8, -61, 0, -82, 82, 144, 72], [0, -168, 1812, 84, 0, 177, -158, -79, 47, -47], [902, -141, 902, 141, -73, -73, -51, 0, 94, 0], [-1611, -213, -1611, 213, 110, 110, 286, 0, 35, 0], [3952, -135, -7904, 0, 26, -52, 73, 73, 0, 17], [2474, 130, -2474, 130, -126, 126, -80, -160, 49, 98], [0, 254, -1253, -127, 0, 4, 158, 79, -115, 115], [-1986, 417, -1986, -417, -131, -131, -303, 0, 22, 0], [-5528, 0, 2764, 269, 0, 0, 0, 118, 42, 42], [543, -297, 0, 594, -213, 0, -231, 231, -124, -62], [-3764, 0, 1882, -51, 38, -19, 0, -393, 22, 22], [261, 344, 261, -344, -33, -33, -18, 0, -27, 0], [-4334, 0, 2167, -12, 120, -60, 0, 456, -188, -188], [-756, 18, 756, 18, 84, -84, -34, -68, -130, -260], [1193, 417, -2386, 0, -181, 362, 165, 165, 0, 71], [-4032, 214, -4032, -214, 150, 150, 351, 0, 153, 0], [0, 524, -2915, -262, 0, -47, -28, -14, -44, 44], [-3658, 150, 3658, 150, 258, -258, -72, -144, 75, 150], [588, -37, 0, 74, 132, 0, 122, -122, 292, 146], [4088, 0, -2044, 144, 64, -32, 0, 400, 74, 74], [-775, 54, -775, -54, 169, 169, 180, 0, 118, 0], [-3246, 0, 1623, 23, -90, 45, 0, -537, -126, -126], [-4318, 60, 8636, 0, -14, 28, -12, -12, 0, -199], [4704, 219, 0, -438, -33, 0, 141, -141, -518, -259], [-3080, 76, -3080, -76, 184, 184, 474, 0, 130, 0], [-4552, 0, 2276, 687, -34, 17, 0, 129, -145, -145], [-2412, -29, 2412, -29, -291, 291, 9, 18, -189, -378], [-708, -366, 1416, 0, 78, -156, -150, -150, 0, 192], [0, 128, 4968, -64, 0, -372, 6, 3, -57, 57], [-6541, 227, -6541, -227, -112, -112, -531, 0, 26, 0], [-2697, 45, 0, -90, 354, 0, 300, -300, 110, 55], [2900, 115, -2900, 115, 498, -498, -281, -562, -285, -570], [65, -177, -130, 0, 192, -384, -429, -429, 0, -193], [2014, 286, 0, -572, 57, 0, 148, -148, 430, 215], [4139, 691, 4139, -691, 24, 24, 797, 0, 108, 0], [0, 242, 1068, -121, 0, 48, -948, -474, 8, -8], [918, 31, -918, 31, 111, -111, -112, -224, -136, -272], [2632, 193, 0, -386, -68, 0, -215, 215, -98, -49], [-3027, -483, 6054, 0, 326, -652, -897, -897, 0, 90], [3744, -205, 3744, 205, -70, -70, -194, 0, -652, 0], [-3467, 122, 6934, 0, 338, -676, 46, 46, 0, 263], [10548, 413, 0, -826, 96, 0, 428, -428, 20, 10], [-5567, -25, 5567, -25, 199, -199, 581, 1162, -68, -136], [0, -72, -8440, 36, 0, -168, 648, 324, -140, 140], [-1193, 63, -1193, -63, 13, 13, -909, 0, 207, 0], [-5166, 0, 2583, -282, -76, 38, 0, -764, 157, 157], [-3078, -141, 6156, 0, 201, -402, 237, 237, 0, -174], [13052, 0, -6526, 183, -166, 83, 0, 3, 476, 476], [0, 400, 3801, -200, 0, 729, 264, 132, -125, 125], [-9332, 0, 4666, -75, 118, -59, 0, 681, 48, 48], [3336, -44, -3336, -44, 324, -324, -324, -648, -58, -116], [-9930, 79, 9930, 79, 213, -213, 92, 184, 22, 44], [0, -1252, -792, 626, 0, 138, -516, -258, 210, -210], [3451, -1083, 3451, 1083, 212, 212, -75, 0, -147, 0], [7699, -84, -7699, -84, -321, 321, -330, -660, -264, -528], [-570, 751, 0, -1502, 351, 0, -44, 44, -508, -254], [-17656, 0, 8828, -435, -120, 60, 0, 326, 372, 372], [-9650, 42, -9650, -42, 169, 169, -66, 0, -721, 0], [9642, 238, 0, -476, 75, 0, -524, 524, -60, -30], [14430, 0, -7215, 405, -334, 167, 0, -81, 17, 17], [0, -548, 13236, 274, 0, -558, 272, 136, 370, -370], [2146, -186, 0, 372, -290, 0, -378, 378, 146, 73], [5419, -363, -10838, 0, 183, -366, -777, -777, 0, -743], [-1196, 828, -1196, -828, -30, -30, -522, 0, -340, 0], [384, 552, 384, -552, -98, -98, -460, 0, 1162, 0], [17228, -375, -17228, -375, -411, 411, -429, -858, -221, -442], [-9758, 366, 9758, 366, 202, -202, -216, -432, 292, 584], [16464, 0, -8232, -9, -736, 368, 0, 1138, 22, 22], [-9228, -492, -9228, 492, -67, -67, 174, 0, -790, 0], [-5592, 0, 2796, 644, -354, 177, 0, -276, 546, 546], [7324, 0, -3662, 1155, -6, 3, 0, 255, -594, -594], [0, -590, 2322, 295, 0, -192, -732, -366, 444, -444], [-2391, -431, 2391, -431, 102, -102, 686, 1372, -345, -690], [2642, -141, -5284, 0, 175, -350, -573, -573, 0, 506], [-13060, -158, -13060, 158, -171, -171, -196, 0, -546, 0], [0, -1248, 5288, 624, 0, -408, -336, -168, 264, -264], [-4556, -1256, 9112, 0, -120, 240, -694, -694, 0, 234], [11257, -205, -11257, -205, -593, 593, 761, 1522, -379, -758], [-6600, 0, 3300, -447, 244, -122, 0, -154, 584, 584], [13911, -105, -27822, 0, 246, -492, -303, -303, 0, 69], [19317, -98, 0, 196, -525, 0, 850, -850, 1002, 501], [12010, 172, 0, -344, 271, 0, 724, -724, -330, -165], [2012, 285, -4024, 0, 120, -240, 961, 961, 0, -681], [3214, 0, -1607, 846, -1354, 677, 0, 612, -371, -371], [1479, -314, -1479, -314, 612, -612, -664, -1328, 87, 174], [14074, -743, 0, 1486, 901, 0, 61, -61, 584, 292], [-14580, -437, 14580, -437, 984, -984, -330, -660, 228, 456], [21248, -758, -21248, -758, 86, -86, 94, 188, -126, -252], [8460, -269, 0, 538, 36, 0, -50, 50, -908, -454], [0, 228, 5108, -114, 0, 764, 2316, 1158, -434, 434], [20646, 0, -10323, -452, 304, -152, 0, 50, 995, 995], [27653, 653, 0, -1306, 284, 0, 269, -269, -148, -74], [-15780, 170, 31560, 0, 415, -830, -176, -176, 0, 802], [64, 340, 64, -340, 40, 40, -288, 0, -68, 0], [0, 516, 8982, -258, 0, -855, -2920, -1460, -40, 40], [-11928, 285, 23856, 0, -29, 58, -741, -741, 0, 585], [0, 958, -21156, -479, 0, -444, -292, -146, -270, 270], [-8053, 576, -8053, -576, 49, 49, 274, 0, 91, 0], [1056, 934, -2112, 0, -154, 308, 254, 254, 0, -616], [16640, 721, -16640, 721, 135, -135, 109, 218, 305, 610], [0, -1234, 3889, 617, 0, -1386, -946, -473, 409, -409]]
